# -*- coding: utf-8 -*-
#
# Copyright (c) 2008 Per Øyvind Karlsen
#
# Written by Per Øyvind Karlsen <peroyvind@mandriva.org>
# Parts of code based on urpmi2smart by Michael Scherer <misc@mandriva.org> 
#
# This file is part of Smart Package Manager.
#
# Smart Package Manager is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# Smart Package Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Smart Package Manager; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import posixpath
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
import os

# be compatible with 2.3
import sys
if sys.version_info < (2, 4):
    from sets import Set as set

from smart.channel import *
from smart import *


URPMI_CONFIG = "/etc/urpmi/urpmi.cfg"
URPMI_MEDIA_DIR = "/var/lib/urpmi/"
URPMI_MIRRORS_CACHE = "/var/cache/urpmi/mirrors.cache"

def fetch_urpmi_cfg(filename):
    if os.path.isfile(URPMI_MIRRORS_CACHE):
        mirrorsfile = open(URPMI_MIRRORS_CACHE)
        mirrorscache = eval(mirrorsfile.read().replace('=>', ':').replace(';',''))
        mirrorsfile.close()
    else:
        mirrorscache = None

    config=[]
    in_block = False
    urpmi_cfg_f=open(filename)
    for line in urpmi_cfg_f.readlines():
        line = line.strip()
        if len(line) <= 0:
            continue
        if line[-1] == "}":
            in_block = False
        if line[-1] == "{":
            in_block = True
            s = line.split(" ")
            if len(s) == 1:
                continue
            config.append(dict())
            config[-1]["name"] = " ".join(s[0:-2]).replace("\\", "")
            if s[-2]:
                config[-1]["url"] = s[-2]
            continue
        if not in_block:
            continue
        if line.find(": ") > 0:
            (key,value) = line.strip().split(": ")
            if key == "mirrorlist":
                if mirrorscache:
                    if mirrorscache.has_key(value):
                        value = mirrorscache[value]
                        config[-1]["url"] = value["list"][0]["url"]
            config[-1][key.strip()] = value
        else:
            config[-1][line.strip()] = True
    urpmi_cfg_f.close()
    return config

def _loadMediaList(filename):

    # The computed aliases we have seen in the given file.
    seen = set()

    urpmi_cfg = fetch_urpmi_cfg(filename)

    for media in urpmi_cfg:
        name = media["name"]
        alias = "urpmisync-%s" % name
        priority = 0
        
        if not media.has_key("url"): 
            continue

        baseurl = media["url"].replace("ssh://", "scp://")

        hdlurl = media["url"] + "/"
        
        if media.has_key("with_hdlist"):
            hdlurl += media["with_hdlist"]
        elif media.has_key("media_info_dir"):
            hdlurl += media["media_info_dir"] + "/synthesis.hdlist.cz"
        else:
            if media.has_key("with-dir"):
                hdlurl += media["with-dir"] + "/"
            hdlurl += "media_info/synthesis.hdlist.cz"
        
        hdlurl = hdlurl.replace("ssh://", "scp://")
        
        seen.add(alias)

        data = {"type": "urpmi",
                "name": name,
                "baseurl": baseurl,
                "hdlurl": hdlurl,
                "disabled": media.has_key("ignore"),
                "removable": media["url"].startswith("cdrom://"),
                "priority": priority}
        # See if creating a channel works.
        try:
            createChannel(alias, data)
        except Error, e:
            iface.error(_("While using %s: %s") % (file.name, e))
        else:
            # Store it persistently, without destroying existing setttings.
            channel = sysconf.get(("channels", alias))
            if channel is not None:
                channel.update(data)
            else:
                channel = data

            sysconf.set(("channels", alias), channel)

    return seen


def syncURPMIChannels(urpmicfg, mediadir, force=None):
    seen = set()

    sysconf.set("sync-urpmi-mediadir", mediadir, soft=True, weak=True)

    # First, the sources.list file.
    if os.path.isfile(urpmicfg):
        seen.update(_loadMediaList(urpmicfg))

    # Delete the entries which were not seen in current files.
    channels = sysconf.get("channels")
    for alias in sysconf.keys("channels"):
        if alias.startswith("urpmisync-") and alias not in seen:
            sysconf.remove(("channels", alias)) 

if not sysconf.getReadOnly():
    if sysconf.get("sync-urpmi-medialist",False):
        iface.debug(_("Trying to sync urpmi channels..."))
        syncURPMIChannels(sysconf.get("urpmi-config", URPMI_CONFIG),
                sysconf.get("urpmi-media-dir", URPMI_MEDIA_DIR))

# vim:ts=4:sw=4:et
